--------------------------------------------------------------------------------
-- CERN (BE-CO-HT)
-- Converter Common Gateware  
-- https://www.ohwr.org/projects/conv-common-gw
--------------------------------------------------------------------------------
--
-- unit name:     Top-level simulation of conv_common_gw
--
-- description:
--
--   
--
--------------------------------------------------------------------------------
-- Copyright (c) 2010-2012 CERN / BE-CO-HT
--------------------------------------------------------------------------------
-- GNU LESSER GENERAL PUBLIC LICENSE
--------------------------------------------------------------------------------
-- This source file is free software; you can redistribute it and/or modify it
-- under the terms of the GNU Lesser General Public License as published by the
-- Free Software Foundation; either version 2.1 of the License, or (at your
-- option) any later version. This source is distributed in the hope that it
-- will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-- See the GNU Lesser General Public License for more details. You should have
-- received a copy of the GNU Lesser General Public License along with this
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.genram_pkg.all;
use work.conv_common_gw_pkg.all;


entity testbench is
end entity testbench;


architecture behav of testbench is

  --============================================================================
  -- Constant declarations
  --============================================================================
  
  -- Clock periods
  constant c_clk_20_per   : time := 50 ns;
  constant c_clk_125_per  : time :=  8 ns;
  constant c_reset_width  : time := 31 ns;
  
  -- DUT configuration (generics):
  constant c_nr_chans     : integer := 6;
  constant c_nr_inv_chans : integer := 4;
  constant c_board_id     : std_logic_vector(31 downto 0) := x"54424c4f";
  constant c_gwvers       : std_logic_vector(7  downto 0) :=x"40";
  
  
  -- Number of I2C masters and slaves for the I2C bus model
  constant c_nr_masters : positive := 1;
  constant c_nr_slaves  : positive := 1;
  --============================================================================
  -- Functions and procedures
  --============================================================================

  --============================================================================
  -- Signal declarations
  --============================================================================
  signal clk_20, clk_125              : std_logic;
  signal clk_125_p, clk_125_n         : std_logic;
  signal rst_n                        : std_logic;

  signal gf_en_n_in                   : std_logic;
  signal burst_en_n_in                : std_logic;
  signal pulse_width_sel_n_in         : std_logic;    
  
  signal pulse_in                     : std_logic_vector(c_nr_chans-1 downto 0);
  signal pulse_front_in               : std_logic_vector(c_nr_chans-1 downto 0);
  signal pulse_rear_in                : std_logic_vector(c_nr_chans-1 downto 0);
  signal pulse_out                    : std_logic_vector(c_nr_chans-1 downto 0);
  signal inv_pulse_in_n               : std_logic_vector(c_nr_inv_chans-1 downto 0);
  signal inv_pulse_out                : std_logic_vector(c_nr_inv_chans-1 downto 0);
  
  signal rst_n_out                    : std_logic;

  signal sscl_in                      : std_logic;
  signal sscl_out                     : std_logic;
  signal sscl_en_out                  : std_logic;
  signal ssda_in                      : std_logic;
  signal ssda_out                     : std_logic;
  signal ssda_en_out                  : std_logic;
  
  signal mscl_in                      : std_logic;
  signal mscl_out                     : std_logic;
  signal mscl_en_out                  : std_logic;
  signal msda_in                      : std_logic;
  signal msda_out                     : std_logic;
  signal msda_en_out                  : std_logic;
  
  signal sw_gp_in                     : std_logic_vector(7  downto 0);
  signal sw_other_in                  : std_logic_vector(31 downto 0);
  signal hwvers_in                    : std_logic_vector(5  downto 0);
  signal rtmm_in                      : std_logic_vector(2  downto 0);
  signal rtmp_in                      : std_logic_vector(2  downto 0);
  
  signal line_front_in                : std_logic_vector(c_nr_chans-1 downto 0);
  signal line_inv_in                  : std_logic_vector(c_nr_inv_chans-1 downto 0);
  signal line_rear_in                 : std_logic_vector(c_nr_chans-1 downto 0);
  signal line_front_fs_in             : std_logic_vector(c_nr_chans-1 downto 0);
  signal line_inv_fs_in               : std_logic_vector(c_nr_inv_chans-1 downto 0);
  signal line_rear_fs_in              : std_logic_vector(c_nr_chans-1 downto 0);
  
  signal i2c_master_ready             : std_logic;
  signal i2c_master_start             : std_logic;
  signal i2c_master_finish            : std_logic;
  signal i2c_master_rdwr              : std_logic;
  signal i2c_master_slv_addr          : std_logic_vector(6 downto 0);
  signal i2c_master_reg_addr          : std_logic_vector(31 downto 0);
  signal i2c_master_send_val          : std_logic_vector(31 downto 0);
  signal i2c_master_rcvd_val          : std_logic_vector(31 downto 0);
 
  type t_i2c_master_in is record
    i2c_master_start                  : std_logic;  
    i2c_master_rdwr                   : std_logic;
    i2c_master_slv_addr               : std_logic_vector(6 downto 0);
    i2c_master_reg_addr               : std_logic_vector(31 downto 0);
    i2c_master_send_val               : std_logic_vector(31 downto 0);
  end record;

  type t_i2c_master_out is record
    i2c_master_ready                  : std_logic;
    i2c_master_finish                 : std_logic;
    i2c_master_rcvd_val               : std_logic_vector(31 downto 0);
  end record;

  signal i2c_m_in                     : t_i2c_master_in;
  signal i2c_m_out                    : t_i2c_master_out;
  
  signal mscl, msda                   : std_logic_vector(c_nr_masters-1 downto 0);
  signal sscl, ssda                   : std_logic_vector(c_nr_slaves-1 downto 0);
  signal scl, sda                     : std_logic;
  ---
  signal buf_byte_cnt: integer;
  
--==============================================================================
--  Procedures
--==============================================================================
--   procedure write_i2c (signal slv_addr: in  std_logic_vector( 6 downto 0);
--                        signal reg_addr: in  std_logic_vector(31 downto 0);
--                        signal send_val: in  std_logic_vector(31 downto 0)) is
--   begin
--     wait until i2c_master_ready = '1';
--     i2c_master_slv_addr  <= slv_addr;
--     i2c_master_reg_addr  <= reg_addr;
--     i2c_master_send_val  <= send_val;
--     i2c_master_start     <= '1';
--     i2c_master_rdwr      <= '0'; --0: write
--     wait until i2c_master_ready = '1';
--     report "write_i2c: OK"; 
--   end procedure write_i2c;
  
  procedure read_i2c (signal i2c_m_in : out t_i2c_master_in;
                      signal i2c_m_out: in  t_i2c_master_out;
                      constant slv_addr : in  std_logic_vector( 6 downto 0);
                      constant reg_addr : in  std_logic_vector(31 downto 0);
                      signal rcvd_val : out std_logic_vector(31 downto 0)) is
  begin
    report "read_i2c: start "; 
    i2c_m_in.i2c_master_slv_addr  <= slv_addr;
    i2c_m_in.i2c_master_reg_addr  <= reg_addr;
    i2c_m_in.i2c_master_start     <= '1';
    i2c_m_in.i2c_master_rdwr      <= '1'; --0: write
    wait for c_clk_20_per;
    i2c_m_in.i2c_master_start     <= '0';
    wait until i2c_m_out.i2c_master_finish = '1';
    rcvd_val             <= i2c_m_out.i2c_master_rcvd_val;
    report "read_i2c: OK "; 
    wait for 50us;
  end procedure read_i2c;
--==============================================================================
--  architecture begin
--==============================================================================
begin

  --============================================================================
  -- Generate clock and reset signals
  --============================================================================
  p_clk_20 : process
  begin
    clk_20 <= '0';
    wait for c_clk_20_per/2;
    clk_20 <= '1';
    wait for c_clk_20_per/2;
  end process;

  p_clk_125 : process
  begin
    clk_125 <= '0';
    wait for c_clk_125_per/2;
    clk_125 <= '1';
    wait for c_clk_125_per/2;
  end process;

  clk_125_p <= clk_125;
  clk_125_n <= not clk_125;
  
  p_rst_n: process
  begin
    rst_n <= '0';
    wait for c_reset_width;
    rst_n <= '1';
    wait;
  end process p_rst_n;
  --============================================================================
  -- Instantiate DUT
  --============================================================================
  cmp_DUT: conv_common_gw
    generic map(
      g_nr_chans                       => c_nr_chans,
      g_nr_inv_chans                   => c_nr_inv_chans,
      g_board_id                       => c_board_id,
      g_gwvers                         => c_gwvers,
      g_pgen_fixed_width               => true
   )
    port map (
      clk_20_i                         => clk_20,
      clk_125_p_i                      => clk_125_p,
      clk_125_n_i                      => clk_125_n,
      rst_n_o                          => rst_n_out,
      gf_en_n_i                        => gf_en_n_in,
      burst_en_n_i                     => burst_en_n_in,
      pulse_width_sel_n_i              => pulse_width_sel_n_in,
      global_ch_oen_o                  => open,
      pulse_front_oen_o                => open,
      pulse_rear_oen_o                 => open,
      inv_oen_o                        => open,
      pulse_i                          => pulse_in,
      pulse_front_i                    => pulse_front_in,
      pulse_rear_i                     => pulse_rear_in,
      pulse_o                          => pulse_out,
      inv_pulse_i_n                    => inv_pulse_in_n,
      inv_pulse_o                      => inv_pulse_out,
      led_pulse_o                      => open,
      led_inv_pulse_o                  => open,
      scl_i                            => scl,
      scl_o                            => sscl_out,
      scl_en_o                         => sscl_en_out,
      sda_i                            => sda,
      sda_o                            => ssda_out,
      sda_en_o                         => ssda_en_out,
      led_i2c_o                        => open,
      vme_sysreset_n_i                 => '1',
      vme_ga_i                         => "11110",
      vme_gap_i                        => '0',
      flash_cs_n_o                     => open,
      flash_sclk_o                     => open,
      flash_mosi_o                     => open,
      flash_miso_i                     => 'Z',
      dac20_din_o                      => open,
      dac20_sclk_o                     => open,
      dac20_sync_n_o                   => open,
      dac125_din_o                     => open,
      dac125_sclk_o                    => open,
      dac125_sync_n_o                  => open,
      sfp_los_i                        => '1',
      sfp_present_i                    => '0',
      sfp_rate_select_o                => open,
      sfp_sda_b                        => open,
      sfp_scl_b                        => open,
      sfp_tx_disable_o                 => open,
      sfp_tx_fault_i                   => '1',
      sw_gp_i                          => sw_gp_in,
      sw_other_i                       => sw_other_in,
      hwvers_i                         => hwvers_in,
      rtmm_i                           => rtmm_in,
      rtmp_i                           => rtmp_in,
      line_front_i                     => line_front_in,
      line_inv_i                       => line_inv_in,
      line_rear_i                      => line_rear_in,
      line_front_fs_i                  => line_front_fs_in,
      line_inv_fs_i                    => line_inv_fs_in,
      line_rear_fs_i                   => line_rear_fs_in,
      thermometer_b                    => open,
      led_syserr_o                     => open,
      bicolor_led_state_i              => (others =>'0'),
      bicolor_led_col_o                => open,
      bicolor_led_line_o               => open,
      bicolor_led_line_oen_o           => open
  );
  
  sscl(0) <= sscl_out when (sscl_en_out = '1') else '1';
  ssda(0) <= ssda_out when (ssda_en_out = '1') else '1';

  --============================================================================
  -- Instantiate I2@ master to access DUT
  --============================================================================

  cmp_i2c_master_and_driver: entity work.i2c_master_and_driver
    port map (
      clk_i                            => clk_20,    
      rst_n_i                          => rst_n_out,
     
--       ready_o                          => i2c_master_ready,
--       start_i                          => i2c_master_start,
--       finish_o                         => i2c_master_finish,
--       rdwr_i                           => i2c_master_rdwr,
--       slv_addr_i                       => i2c_master_slv_addr,
--       reg_addr_i                       => i2c_master_reg_addr,
--       send_val_i                       => i2c_master_send_val,
--       rcvd_val_o                       => i2c_master_rcvd_val,

      ready_o                          => i2c_m_out.i2c_master_ready,
      start_i                          => i2c_m_in.i2c_master_start,
      finish_o                         => i2c_m_out.i2c_master_finish,
      rdwr_i                           => i2c_m_in.i2c_master_rdwr,
      slv_addr_i                       => i2c_m_in.i2c_master_slv_addr,
      reg_addr_i                       => i2c_m_in.i2c_master_reg_addr,
      send_val_i                       => i2c_m_in.i2c_master_send_val,
      rcvd_val_o                       => i2c_m_out.i2c_master_rcvd_val,
      
      scl_i                            => scl,
      scl_o                            => mscl_out,
      scl_oen                          => mscl_en_out,
      sda_i                            => sda,
      sda_o                            => msda_out,
      sda_oen                          => msda_en_out
  );
   
  -- Tri-state buffers on the I2C lines
  mscl(0) <= mscl_out when (mscl_en_out = '0') else '1';
  msda(0) <= msda_out when (msda_en_out = '0') else '1';
  
  ------------------------------------------------------------------------------
  -- Bus model instantiation and connection to master and slaves
  ------------------------------------------------------------------------------
  cmp_i2c_bus : entity work.i2c_bus_model
    generic map(
      g_nr_masters => c_nr_masters,
      g_nr_slaves  => c_nr_slaves
    )
    port map (
      mscl_i => mscl,
      msda_i => msda,
      sscl_i => sscl,
      ssda_i => ssda,
      scl_o  => scl,
      sda_o  => sda
    );
    
  --============================================================================
  -- processes to provide stimulus and checks
  --============================================================================
  -- stimulus for configuration
  p_stim_config : process
  begin
    wait for 2 us; -- does not work without this wait, no idea way
    gf_en_n_in              <= '1';-- enable glitch filter
    burst_en_n_in           <= '1';-- enable burt mode 
    pulse_width_sel_n_in    <= '1';-- set pulse width to 0:250ns | 1:1.2us
 
    sw_gp_in(0)             <= '0';
    sw_gp_in(1)             <= '0';
    sw_gp_in(7 downto 2)    <= (others => '1');
  
    sw_other_in             <= (others => '0');
    hwvers_in               <= "111100";
    rtmm_in                 <= (others => '0');
    rtmp_in                 <= (others => '0');
  
  end process p_stim_config;
  
  -- stimulus for inputs with output check
  p_stim_io : process
  begin
    pulse_in              <= (others => '0');
    pulse_front_in        <= (others => '0');
    pulse_rear_in         <= (others => '0');
    inv_pulse_in_n        <= (others => '0');
    inv_pulse_out         <= (others => '0');
    line_front_in           <= (others => '0');
    line_inv_in             <= (others => '0');
    line_rear_in            <= (others => '0');
    line_front_fs_in        <= (others => '0');
    line_inv_fs_in          <= (others => '0');
    line_rear_fs_in         <= (others => '0');

    
    wait for 250 us; --min period is 240us
    
    pulse_in              <= (others => '1');
    wait for 1.2 us; 
    
    pulse_in              <= (others => '0');
  end process p_stim_io;
    
  -- i2c access
  p_stim_i2c : process
  begin
    i2c_m_in.i2c_master_start    <= '0';
    i2c_m_in.i2c_master_rdwr     <= '0'; --mst_fsm_op
    i2c_m_in.i2c_master_slv_addr <= "1011110";
    i2c_m_in.i2c_master_reg_addr <= (others => '0');
    i2c_m_in.i2c_master_send_val <= (others => '1');
    
    wait until rst_n = '1';
    
    report "started stimulus";
    wait for 50us;
    
    read_i2c (i2c_m_in, i2c_m_out, "1011110",x"0000_0030", i2c_master_rcvd_val );
    read_i2c (i2c_m_in, i2c_m_out, "1011110",x"0000_0034", i2c_master_rcvd_val );
    read_i2c (i2c_m_in, i2c_m_out, "1011110",x"0000_0038", i2c_master_rcvd_val );
    read_i2c (i2c_m_in, i2c_m_out, "1011110",x"0000_002c", i2c_master_rcvd_val );  
    
  end process p_stim_i2c;
end architecture behav;
--==============================================================================
--  architecture end
--==============================================================================


